@@ -4,6 +4,9 @@ import logging
 import os
 import time
 from collections import OrderedDict
+
+from requests.auth import AuthBase
+
 try:
     from urllib.parse import urlencode
 except ImportError:
@@ -23,6 +26,25 @@ from volcengine.util.Util import *
 from volcengine import VERSION
 
 
+class VolcAuth(AuthBase):
+    def __init__(self, client, request):
+        # setup any auth-related data here
+        self.client = client
+        self.request = request
+
+    def __call__(self, r):
+        self.request.body = r.body
+        if "Content-Type" in r.headers:
+            self.request.headers["Content-Type"] = r.headers["Content-Type"]
+        if "Content-Length" in r.headers:
+            self.request.headers["Content-Length"] = r.headers["Content-Length"]
+        SignerV4.sign(self.request, self.client.service_info.credentials)
+        for k in self.request.headers:
+            v = self.request.headers[k]
+            r.headers[k] = v
+        return r.headers
+
+
 class Service(object):
     def __init__(self, service_info, api_info):
         self.service_info = service_info
@@ -63,7 +85,6 @@ class Service(object):
                     if 'sk' in j:
                         self.service_info.credentials.set_sk(j['sk'])
 
-
     def set_ak(self, ak):
         self.service_info.credentials.set_ak(ak)
 
@@ -129,6 +150,25 @@ class Service(object):
         else:
             raise Exception(resp.text)
 
+    def request(self, api, params, data, files=None, reqConfig=None):
+
+        if not (api in self.api_info):
+            raise Exception("no such api")
+        api_info = self.api_info[api]
+
+        r = self.prepare_request(api_info, params)
+        if reqConfig is not None:
+            reqConfig(r)
+        url = r.build()
+
+        resp = self.session.request(api_info.method, url, data=data, files=files,
+                                    timeout=(self.service_info.connection_timeout, self.service_info.socket_timeout),
+                                    auth=VolcAuth(self, r))
+        if resp.status_code == 200:
+            return resp.text
+        else:
+            raise Exception(resp.text)
+
     def json(self, api, params, body):
         if not (api in self.api_info):
             raise Exception("no such api")
