@@ -14,5 +14,5 @@ if __name__ == '__main__':
     params['Scene'] = "license"
     params['StoreUri'] = "xx"
 
-    resp = imagex_service.get_image_ocr(params)
+    resp = imagex_service.get_image_ocr_v2(params)
     print(resp)
