@@ -11,7 +11,8 @@ if __name__ == '__main__':
 
     params = dict()
     params['ServiceId'] = 'imagex service id'
-
+    params['SkipMeta'] = False
+    params['SkipCommit'] = False
     file_paths = ['image file path 1']
     resp = imagex_service.upload_image(params, file_paths)
     print(resp)
