@@ -29,6 +29,9 @@ class Uploader:
         self.queue = queue.Queue()
         self.queue_lock = threading.Lock()
 
+        self.successOids = []
+        self.results = []
+
         for i in range(len(store_infos)):
             self.queue.put(i)
 
@@ -44,21 +47,33 @@ class Uploader:
             store_info = self.store_infos[idx]
             file_paths_or_bytes = self.file_paths_or_bytes[idx]
 
-            if isinstance(file_paths_or_bytes, bytes):
-                self.upload_by_host(store_info, file_paths_or_bytes)
-            elif isinstance(file_paths_or_bytes, str):
-                file_path = file_paths_or_bytes
-                file_size = os.path.getsize(file_path)
-                data = open(file_path, "rb")
-                if file_size < MinChunkSize:
-                    self.upload_by_host(store_info, data.read())
-                elif file_size > LargeFileSize:
-                    self.chunk_upload(store_info, data, file_size, True)
+            try:
+                if isinstance(file_paths_or_bytes, bytes):
+                    self.upload_by_host(store_info, file_paths_or_bytes)
+                elif isinstance(file_paths_or_bytes, str):
+                    file_path = file_paths_or_bytes
+                    file_size = os.path.getsize(file_path)
+                    data = open(file_path, "rb")
+                    if file_size < MinChunkSize:
+                        self.upload_by_host(store_info, data.read())
+                    elif file_size > LargeFileSize:
+                        self.chunk_upload(store_info, data, file_size, True)
+                    else:
+                        self.chunk_upload(store_info, data, file_size, False)
+                    data.close()
                 else:
-                    self.chunk_upload(store_info, data, file_size, False)
-                data.close()
-            else:
-                raise Exception("Uploader only accept bytes or path data")
+                    raise Exception("Uploader only accept bytes or path data")
+                self.successOids.append(store_info['StoreUri'])
+                self.results.append({
+                    'Uri': store_info['StoreUri'],
+                    'UriStatus': 2000,
+                })
+            except Exception as e:
+                self.results.append({
+                    'Uri': store_info['StoreUri'],
+                    'UriStatus': 2001,
+                    'Error': e,
+                })
 
     @retry(tries=3, delay=1, backoff=2)
     def upload_by_host(self, store_info, img_data):
@@ -190,6 +205,7 @@ class ImageXService(Service):
             'SessionKey': params.get('SessionKey', ''),
             'UploadNum': len(file_paths),
             'StoreKeys': params.get('StoreKeys', []),
+            'Overwrite': str(params.get('Overwrite', False)),
         }
         resp = self.apply_upload(apply_upload_request)
         if 'Error' in resp['ResponseMetadata']:
@@ -206,13 +222,19 @@ class ImageXService(Service):
 
         session_key = addr['SessionKey']
         host = addr['UploadHosts'][0]
-        self.do_upload(file_paths, host, addr['StoreInfos'])
+        uploader = self.do_upload(file_paths, host, addr['StoreInfos'])
+        if len(uploader.successOids) == 0:
+            raise Exception("no file uploaded")
 
+        if params.get('SkipCommit', False):
+            return {'Results': uploader.results}
         commit_upload_request = {
             'ServiceId': params['ServiceId'],
+            'SkipMeta': params.get('SkipMeta', False)
         }
         commit_upload_body = {
             'SessionKey': session_key,
+            'SuccessOids': uploader.successOids,
             'Functions': params.get('Functions', []),
             'OptionInfos': params.get('OptionInfos', [])
         }
@@ -250,13 +272,19 @@ class ImageXService(Service):
 
         session_key = addr['SessionKey']
         host = addr['UploadHosts'][0]
-        self.do_upload(img_datas, host, addr['StoreInfos'])
+        uploader = self.do_upload(img_datas, host, addr['StoreInfos'])
+        if len(uploader.successOids) == 0:
+            raise Exception("no file uploaded")
 
+        if params.get('SkipCommit', False):
+            return {'Results': uploader.results}
         commit_upload_request = {
             'ServiceId': params['ServiceId'],
+            'SkipMeta': params.get('SkipMeta', False)
         }
         commit_upload_body = {
             'SessionKey': session_key,
+            'SuccessOids': uploader.successOids,
             'Functions': params.get('Functions', []),
             'OptionInfos': params.get('OptionInfos', [])
         }
@@ -273,9 +301,9 @@ class ImageXService(Service):
             thread = threading.Thread(target=uploader.async_upload)
             thread.start()
             threads.append(thread)
-
         for i in range(UPLOAD_THREADS):
             threads[i].join()
+        return uploader
 
     def get_upload_auth_token(self, params):
         apply_token = self.get_sign_url('ApplyImageUpload', params)
@@ -289,7 +317,7 @@ class ImageXService(Service):
         else:
             return base64.b64encode(data.decode('utf-8'))
 
-    def get_upload_auth(self, service_ids, expire=60 * 60, key_ptn=''):
+    def get_upload_auth(self, service_ids, expire=60 * 60, key_ptn='', tag=dict):
         apply_res = []
         commit_res = []
         if len(service_ids) == 0:
@@ -307,6 +335,8 @@ class ImageXService(Service):
             Statement.new_allow_statement(
                 ['ImageX:CommitImageUpload'], commit_res)
         ])
+        for k, v in tag.items():
+            inline_policy.statements.append(Statement.new_allow_statement([k], [str(v)]))
         return self.sign_sts2(inline_policy, expire)
 
     def delete_images(self, service_id, uris):
@@ -381,6 +411,13 @@ class ImageXService(Service):
         res_json = json.loads(json.dumps(res))
         return res_json
 
+    def imagex_request(self, action, params, body, files):
+        res = self.request(action, params, body, files)
+        if res == '':
+            raise Exception("%s: empty response" % action)
+        res_json = json.loads(json.dumps(res))
+        return res_json
+
     def fetch_image_url(self, request):
         res = self.imagex_post('FetchImageUrl', request, json.dumps(request))
         if res == '':
@@ -399,8 +436,17 @@ class ImageXService(Service):
             raise Exception(res_json['ResponseMetadata'])
         return res_json['Result']
 
-    def get_image_ocr(self, params):
-        res = self.post('GetImageOCR', params, {})
+    def update_image_storage_ttl(self, request):
+        res = self.imagex_post('UpdateImageStorageTTL', {}, json.dumps(request))
+        if res == '':
+            raise Exception("empty response")
+        res_json = json.loads(res)
+        if 'Error' in res_json['ResponseMetadata']:
+            raise Exception(res_json['ResponseMetadata'])
+        return res_json['Result']
+
+    def get_image_ocr_v2(self, params):
+        res = self.post('GetImageOCRV2', params, {})
         if res == '':
             raise Exception("%s: empty response" % 'GetImageOCR')
         res_json = json.loads(res)
@@ -459,6 +505,20 @@ class ImageXService(Service):
         res_json = json.loads(res)
         return res_json
 
+    def get_image_enhance_result_with_data(self, input, data):
+        params = {
+            "ServiceId": input["ServiceId"]
+        }
+        res = self.imagex_request('GetImageEnhanceResultWithData', params, body={
+            'Input': json.dumps(input),
+        }, files={
+            'Data': ('img', data),
+        })
+        if res == '':
+            raise Exception("%s: empty response" % 'GetImageEnhanceResult')
+        res_json = json.loads(res)
+        return res_json
+
     def get_image_bg_fill_result(self, params):
         body = {
             'ServiceId': params['ServiceId'],
@@ -475,6 +535,23 @@ class ImageXService(Service):
         res_json = json.loads(res)
         return res_json
 
+    def get_image_duplicate_detection(self, body):
+        query = {
+            'ServiceId': body['ServiceId'],
+        }
+        res = self.imagex_post('GetImageDuplicateDetection', query, json.dumps(body))
+        if res == '':
+            raise Exception("%s: empty response" % 'GetImageDuplicateDetection')
+        res_json = json.loads(res)
+        return res_json
+
+    def get_image_duplicate_task_status(self, query):
+        res = self.imagex_get('GetDedupTaskStatus', query)
+        if res == '':
+            raise Exception("%s: empty response" % 'GetDedupTaskStatus')
+        res_json = json.loads(res)
+        return res_json
+
     def get_image_comic_result(self, params):
         body = {
             'ServiceId': params['ServiceId'],
@@ -508,6 +585,16 @@ class ImageXService(Service):
         res_json = json.loads(res)
         return res_json
 
+    def get_image_style_result(self, args):
+        query = {
+            'ServiceId': args['ServiceId'],
+        }
+        res = self.imagex_post('GetImageStyleResult', query, json.dumps(args))
+        if res == '':
+            raise Exception("%s: empty response" % 'GetImageStyleResult')
+        res_json = json.loads(res)
+        return res_json
+
     def describe_imagex_domain_traffic_data(self, query):
         res = self.imagex_get('DescribeImageXDomainTrafficData', query)
         if res == '':
